# xDevs.com BBCode/Textile URL generator 
#    Changes localfile absolute image path into repository image storage URL
#    http://xdevs.com/guide/markup_gen
import sys
import time
from math import log10

dmm_val = 1.0
dmm_temp = 36.6

class dmm():
    global dmm_val 
    data = ""
    temp = 36.6
    cnti = 12
    
    def __init__(self,gpib,refhp,name):
        self.gpib = gpib
        #self.inst = Gpib.Gpib(0,self.gpib, timeout=60) # 3458A GPIB Address = self.gpib
        self.refhp = refhp
        self.name = name
        self.init_inst()

    def init_inst(self):
        # Setup HP 3458A
        self.inst.write ("PRESET NORM")
        self.inst.write ("OFORMAT ASCII")
        self.inst.write ("FUNC DCV,AUTO")
        self.inst.write ("TARM HOLD")
        self.inst.write ("TRIG LINE")
        self.inst.write ("NPLC 100")
        self.inst.write ("AZERO ON")
        self.inst.write ("NRDGS 1,AUTO")
        self.inst.write ("MEM OFF")
        self.inst.write ("END ALWAYS")
        self.inst.write ("NDIG 9")
        
    def read_data(self,cmd):
        data_float = 0.0
        data_str = ""
        self.inst.write(cmd)
        
        try:
            with Timeout(20):
                data_str = self.inst.read()
        except Timeout.Timeout:
            print ("Timeout exception from dmm %s on read_data() inst.read()\n" % self.name)
            return (0,float(0))
        #print ("Reading from dmm %s = %s" % (self.name,data_str))
        try:
            data_float = float(data_str)
        except ValueError:
            print("Exception thrown by dmm %s on read_data() - ValueError = %s\n" % (self.name,data_str))
            return (0,float(0)) # Exception on float conversion, 0 = error
        return (1,data_float) # Good read, 1 = converted to float w/o exception
        
    def get_temp(self):
        global dmm_temp
        print("Reading DMM temp")
        self.inst.write("TARM SGL,1")
        self.temp_status_flag,temp = self.read_data("TEMP?")
        if (self.temp_status_flag):
            self.temp = temp
        dmm_temp = 37.5
        return self.temp
        
    def get_temp_status(self):
        return self.temp_status_flag

    def get_data(self):
        global dmm_val
        self.status_flag,data = self.read_data("TARM SGL,1")
        if (self.status_flag):
            self.data = data
            dmm_val = float(data)
            self.ppm = ((float(self.data) / self.refhp)-1)*1E6
        return self.data
        
    def exec_acal(self):
        sys.stdout.write ("ACAL procedure start, please wait 12 minutes ")
        #self.inst.write("ACAL ALL")
        for cnt in range(0 ,48):        # wait 720 seconds, print dot every 15s
            sys.stdout.write (".")
            time.sleep(0.01)
        print("\r\nACAL procedure done...")
    
    def exec_idn(self):
        self.inst.write ("END ALWAYS")
        self.inst.write ("ID?")
        if (self.data == "HP3458A"):
            print ("\r\n%s detected..." % data)
        else:
            print ("\r\nNo DMM present, exiting")
            quit()

    def get_data_status(self):
        return self.status_flag

with open('callog.txt','wb') as b:
    kei2400 = Gpib.Gpib(0,24)
    kei2400.clear()

    cal_dcv_set = [-0.2, 0, 0.2, 0, -2, 0, 2, 0, -20, 0, 20, 0, -200, 0, 200, 0]
    cal_dcv_rng = [0.2, 0.2, 0.2, 0.2, 2, 2, 2, 2, 20, 20, 20, 20, 200, 200, 200, 200]
    cal_dcv_str = ["-0.2 m"," -Zero"," 0.2 m"," +Zero","   -2 "," -Zero","    2 "," -Zero","  -20 "," +Zero","   20 "," -Zero"," -200 "," +Zero","  200 "," +Zero"]
    cal_dci_set = [-1E-6, 0, 1E-6, 0, -10E-6, 0, 10E-6, 0, -100E-6, 0, 100E-6, 0, -1E-3, 0, 1E-3, 0, -10E-3, 0, 10E-3, 0, -100E-3, 0, 100E-3, 0, -1, 0, 1, 0]
    cal_dci_rng = [1E-6, 1E-6, 1E-6, 1E-6, 10E-6, 10E-6, 10E-6, 10E-6, 100E-6, 100E-6, 100E-6, 100E-6, 1E-3, 1E-3, 1E-3, 1E-3, 10E-3, 10E-3, 10E-3, 10E-3, 100E-3, 100E-3, 100E-3, 100E-3, 1, 1, 1, 1]
    test_dcv_set = [0, -0.002, 0.002, -0.02, 0.02, -0.2, 0.2, -1, 1, -2, 2, -10, 10, -15, 15, -20, 20, -100, 100, -200, 200, 0]
    test_dci_set = [0, -1E-9, 1E-9, -10E-9, 10E-9, -100E-9, 100E-9, -1E-6, 1E-6, -10E-6, 10E-6, -100E-6, 100E-6, -1E-3, 1E-3, -4E-3, 4E-3, -10E-3, 10E-3, -20E-3, 20E-3, -100E-3, 100E-3, -500E-3, 500E-3, 1, -1, 0]
    test_dci_str = ["Zero  ","  -1 n", "   1 n", " -10 n", "  10 n", "-100 n", " 100 n", "  -1 u", "   1 u", " -10 u", "  10 u", "-100 u", " 100 u", "  -1 m", "   1 m", "  -4 m", "   4 m", " -10 m", "  10 m", " -20 m", "  20 m", "-100 m", " 100 m", "-500 m", " 500 m", "  -1  ", "   1  ", "Zero  "]
    kei2400_idn = "KEITHLEY INSTRUMENTS INC."
    kei2400_mdl = " MODEL 2400"
    orig_val = []
    kei_error = 0
    plc = 60 #60 Hz
    cnt = 0
    samples = 0
    nplc = 0
    val = 3.423
    meas = 0.0
    temp = 36.6
    
    for cnt in range(30):
        orig_val.append(0)
    
    sys.stdout.write("Keithley 2400 calibration tool \r\n  Using NI GPIB adapter with next instruments config: \r\n * GPIB 20 : Keithley 2400 SMU \r\n * GPIB 3 : HP 3458A 8.5-digit DMM\r\n\r\n")
    
    kei2400.write ("*IDN?")
    idn_str = "KEITHLEY INSTRUMENTS INC., MODEL 2400, A0542241, 24124/42411 /C32/C01"
    
    idmfg = idn_str.split(",")
    if (kei2400_idn == idmfg[0]):
        if (kei2400_mdl == idmfg[1]):
            sys.stdout.write("Keithley Model 2400 - detected, S/N %s, Version: %s\r\n" % (idmfg[2], idmfg[3]))
        else:
            print ("Incorrect Model! Check GPIB address. Calibration cancelled.")    
            quit()
    else:
        print ("No Keithley instrument detected. Check GPIB address. Calibration cancelled.")
        quit()
    
    kei2400.write ("*CLS")
    kei2400.write ("*ESE 1;*SRE 32")
    kei2400.write ("*RST")
    kei2400.write (":SENS:FUNC:CONC OFF")
    kei2400.write (":SENS:FUNC 'VOLT:DC'")
    kei2400.write (":SOUR:FUNC VOLT")
    kei2400.write (":OUTP:STAT OFF")
    
    kei2400 "\r\nReading current calibration data from Model 2400\r\n"
    
    for cnt in range (0, 4):
        kei2400.write ":SOUR:VOLT:RANGE %3.1f" % cal_dcv_rng[cnt*4]
        kei2400.write ":CAL:PROT:SENS:DATA?"
        kdata = kei2400.read()
        print kdata
        kei2400.write ":CAL:PROT:SOUR:DATA?"
        kdata = kei2400.read()
        print kdata
        cnt = cnt + 1
    
    kei2400.write ":CAL:PROT:LOCK"
    kei2400.write ":SENS:FUNC 'CURR:DC'"
    kei2400.write ":SOUR:FUNC CURR"
    kei2400.write ":CAL:PROT:CODE 'KI002400'"
    
    for cnt in range (0, 7):
        kei2400.write ":SOUR:CURR:RANGE %3.1e" % cal_dci_rng[cnt*4]
        kei2400.write ":CAL:PROT:SENS:DATA?"
        kdata = kei2400.read()
        print kdata
        kei2400.write ":CAL:PROT:SOUR:DATA?"
        kdata = kei2400.read()
        print kdata
        cnt = cnt + 1
    
    dmm = dmm(3,dmm_val,"3458A") # GPIB 3
    dmm.exec_idn()
    
    dmm.get_temp()
    print ("TEMP = %f C" % dmm_temp)
    
    kei2400.write ":SYST:BEEP:IMM 500, 5"
    raw_input("Press Enter to continue with calibration...")
    
    dmm.exec_acal()
    
    print ("Connect DMM volts input to Model 2400 INPUT/OUTPUT jacks.")
    
    kei2400.write ("*RST")
    kei2400.write (":SOUR:FUNC VOLT")
    kei2400.write (":SENS:CURR:PROT 0.1")
    kei2400.write (":SENS:CURR:RANG 0.1")
    kei2400.write (":SOUR:VOLT:PROT:LEV MAX")
    kei2400.write (":SYST:RSEN OFF")
    kei2400.write (":CAL:PROT:CODE 'KI002400'")
    kei2400.write (":OUTP:STAT ON")
    
    sys.stdout.write("Starting DCV calibration...\r\n\r\n")
    
    for cnt in range (0, len(cal_dcv_set)):
        #sys.stdout.write(" Calibration step %d " % cnt)
        
        kei2400.write ":SOUR:VOLT:RANGE %3.1f" % cal_dcv_rng[cnt]
        kei2400.write ":SOUR:VOLT %3.1f" % cal_dcv_set[cnt]
        
        val = 0 
        for samples in range (0, 10):
            dmm.get_data()
            val += dmm_val
        val = val / 10
        
        orig_val[cnt] = val
        sys.stdout.write("DCV Calibration step %sV : %.9E VDC " % (cal_dcv_str[cnt], val))
        
        kei2400.write ":CAL:PROT:SOUR %3.8f;*OPC" % val
        kei2400.write "*STB?"
        kei2400.write "*ESR?"
        kei2400.write ":SYST:ERR?"
        kei_error = kei2400.read()
        if (kei_error != 0):
            print "Keithley 2400 error : %d" % int(kei_error)
        kei2400.write ":CAL:PROT:SENS %3.8f;*OPC" % val
        kei2400.write "*STB?"
        kei2400.write "*ESR?"
        kei2400.write ":SYST:ERR?"
        kei_error = kei2400.read()
        if (kei_error != 0):
            print "Keithley 2400 error : %d" % int(kei_error)
        
        if (cnt % 2 == 0):
            ppms = ((val / (cal_dcv_set[cnt] + 1E-16)) - 1) * 1E6
            sys.stdout.write(" Delta: %.3f ppm\r\n" % ppms)
        else:
            sys.stdout.write("\r\n")
        cnt = cnt + 1
  
    sys.stdout.write("\r\n")
    sys.stdout.write("Connect DMM current input to 2400 INPUT/OUTPUT jacks. \r\n\r\n")
    
    kei2400.write ":SYST:BEEP:IMM 1000, 5"
    raw_input("Press Enter to continue...")
    
    sys.stdout.write("Starting DCI calibration...\r\n\r\n")
    
    kei2400.write ":SOUR:FUNC CURR"
    kei2400.write ":SENS:VOLT:PROT 20"
    kei2400.write ":SENS:VOLT:RANG 20"
    kei2400.write ":OUTP:STAT ON"
    
    for cnt in range (0, len(cal_dci_set)):
        #sys.stdout.write(" Calibration step %d " % cnt)
        sys.stdout.write("DCI Calibration step  %02d (%.9E) : %.9E ADC \r\n" % (cnt, cal_dci_set[cnt], val))
        
        kei2400.write ":SOUR:CURR:RANGE %3.1e" % cal_dci_rng[cnt]
        kei2400.write ":SOUR:CURR %3.1e" % cal_dci_set[cnt]
        
        val = 0 
        for samples in range (0, 10):
            dmm.get_data()
            val += dmm_val
        val = val / 10
        
        kei2400.write ":CAL:PROT:SOUR %3.8e;*OPC" % val
        kei2400.write ":SYST:ERR?"
        kei_error = kei2400.read()
        if (kei_error != 0):
            print "Keithley 2400 error : %d" % int(kei_error)
        kei2400.write "*STB?"
        kei2400.write "*ESR?"
        kei2400.write ":CAL:PROT:SENS %3.8e;*OPC" % val
        kei2400.write ":SYST:ERR?"
        kei_error = kei2400.read()
        if (kei_error != 0):
            print "Keithley 2400 error : %d" % int(kei_error)
        kei2400.write "*STB?"
        kei2400.write "*ESR?"
        
        cnt = cnt + 1
    
    #        name = "3458A_NPLC%s_DCV%s.log" % (nplc_set[cx],vdc_set[cnv])
    #        file = open(name, 'w+')
    #        for si in range (0, int(samples)):
    #            meas = meas + 0.1
    #            file.write("%f;\r" % meas)
    #        sys.stdout.write("%7.f " % meas)
    #        file.close()
    #        meas = 0
    #    sys.stdout.write("| ppm |")
    #    cnv = cnv + 1
    
    kei2400.write time.strftime(":CAL:PROT:DATE %Y, %m, %d")
    kei2400.write ":CAL:PROT:NDUE 2016, 09, 01"
    kei2400.write ":CAL:PROT:SAVE"
    kei2400.write ":CAL:PROT:LOCK"
    
    quit()
    
    sys.stdout.write("\r\n\r\nPerformance DCI test\r\n")
    
    sys.stdout.write("3458A TEMP? = %04.1f \r\n" % (temp))
    
    min = 0.0
    max = 0.0
    ppm = 10.042
    
    for cnt in range (0, len(test_dci_set)):
        for samples in range (0, 30):
            val = 2.000
            
        sys.stdout.write(" Verification step %sA : %.9E ADC [min %E ADC, max %E ADC, deviation %6.2f ppm]\r\n" % (test_dci_str[cnt], val, min, max, ppm))
        cnt = cnt + 1

    sys.stdout.write("\r\nConnect DMM VOLTAGE input to 2400 INPUT/OUTPUT jacks. \r\n\r\n")
    
    print ":SYST:BEEP:IMM 1000, 5"
    raw_input("Press Enter to continue...")
        
    sys.stdout.write("\r\n\r\nPerformance DCV test\r\n")
    
    sys.stdout.write("3458A TEMP? = %04.1f \r\n" % (temp))
    
    for cnt in range (0, len(test_dcv_set)):
        for samples in range (0, 30):
            val = 2.000
            
        sys.stdout.write(" Verification step  %02d (%.9E) : %.9E VDC [min %E VDC, max %E VDC, deviation %6.2f ppm]\r\n" % (cnt, test_dcv_set[cnt], val, min, max, ppm))
        cnt = cnt + 1
